"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sendAoTransfer = exports.checkArPermissions = void 0;
const arweave_1 = __importDefault(require("arweave"));
const isString_1 = __importDefault(require("lodash/isString"));
const types_1 = require("../types");
const util_1 = require("../utils/util");
const api_1 = require("../api");
const sha256_1 = __importDefault(require("crypto-js/sha256"));
const aoconnect_1 = require("@permaweb/aoconnect");
const arseeding_arbundles_1 = require("arseeding-arbundles");
const options = {
    host: 'arweave.net',
    port: 443,
    protocol: 'https',
    timeout: 20000,
    logging: false // Enable network request logging
};
const defaultAOConfig = {
    CU_URL: 'https://cu.ao-testnet.xyz',
    MU_URL: 'https://mu.ao-testnet.xyz',
    GATEWAY_URL: 'https://g8way.io:443'
};
// TODO: to fix arConnect return result and interface
var ERRORS;
(function (ERRORS) {
    ERRORS["PLEASE_INSTALL_ARCONNECT"] = "PLEASE_INSTALL_ARCONNECT";
    ERRORS["ACCESS_ADDRESS_PERMISSION_NEEDED"] = "ACCESS_ADDRESS_PERMISSION_NEEDED";
    ERRORS["ACCESS_PUBLIC_KEY_PERMISSION_NEEDED"] = "ACCESS_PUBLIC_KEY_PERMISSION_NEEDED";
    ERRORS["SIGNATURE_PERMISSION_NEEDED"] = "NEED_SIGNATURE_PERMISSION";
    ERRORS["SIGN_TRANSACTION_PERMISSION_NEEDED"] = "SIGN_TRANSACTION_PERMISSION_NEEDED";
    ERRORS["SIGNATURE_FAILED"] = "SIGNATURE_FAILED";
    ERRORS["TRANSACTION_POST_ERROR"] = "TRANSACTION_POST_ERROR";
    ERRORS["ACCESS_PUBLIC_KEY_FAILED"] = "ACCESS_PUBLIC_KEY_FAILED";
})(ERRORS || (ERRORS = {}));
const checkArPermissions = async (permissions) => {
    let existingPermissions = [];
    permissions = (0, isString_1.default)(permissions) ? [permissions] : permissions;
    try {
        existingPermissions = await window.arweaveWallet.getPermissions();
    }
    catch {
        throw new Error(ERRORS.PLEASE_INSTALL_ARCONNECT);
    }
    if (permissions.length === 0) {
        return;
    }
    if (permissions.some(permission => {
        return !existingPermissions.includes(permission);
    })) {
        await window.arweaveWallet.connect(permissions);
    }
};
exports.checkArPermissions = checkArPermissions;
const signMessageAsync = async (debug, arJWK, address, messageData) => {
    const arweave = arweave_1.default.init(options);
    const msgDataBuffer = Buffer.from(messageData, 'utf-8');
    let arOwner = '';
    let signatureB64url = '';
    // web
    if (arJWK === 'use_wallet') {
        try {
            await (0, exports.checkArPermissions)('ACCESS_PUBLIC_KEY');
        }
        catch {
            throw new Error(ERRORS.ACCESS_PUBLIC_KEY_PERMISSION_NEEDED);
        }
        try {
            // TODO: wait arweave-js update arconnect.d.ts
            arOwner = await (window.arweaveWallet).getActivePublicKey();
        }
        catch {
            throw new Error(ERRORS.ACCESS_PUBLIC_KEY_FAILED);
        }
        try {
            await (0, exports.checkArPermissions)('SIGNATURE');
        }
        catch {
            throw new Error(ERRORS.SIGNATURE_PERMISSION_NEEDED);
        }
        const algorithm = {
            name: 'RSA-PSS',
            saltLength: 32
        };
        if (window.arweaveWallet.signMessage !== undefined) {
            try {
                const signature = await window.arweaveWallet.signMessage(msgDataBuffer, { hashAlgorithm: 'SHA-256' });
                const buf = new Uint8Array(Object.values(signature));
                signatureB64url = arweave_1.default.utils.bufferTob64Url(buf);
            }
            catch {
                throw new Error(ERRORS.SIGNATURE_FAILED);
            }
        }
        else {
            try {
                const hash = (0, sha256_1.default)(messageData);
                const signature = await (window.arweaveWallet).signature((0, util_1.hexToUint8Array)(hash.toString()), algorithm);
                const buf = new Uint8Array(Object.values(signature));
                signatureB64url = arweave_1.default.utils.bufferTob64Url(buf);
            }
            catch {
                throw new Error(ERRORS.SIGNATURE_FAILED);
            }
        }
        // node
    }
    else {
        const hash = (0, sha256_1.default)(messageData);
        const buf = await arweave.crypto.sign(arJWK, (0, util_1.hexToUint8Array)(hash.toString()), {
            saltLength: 32
        });
        arOwner = arJWK.n;
        signatureB64url = arweave_1.default.utils.bufferTob64Url(buf);
    }
    return `${signatureB64url},${arOwner}`;
};
// TODO: only support browser yet
const sendAoTransfer = async (process, recipient, amount, addTags = [], replaceTags) => {
    const ao = (0, aoconnect_1.connect)(defaultAOConfig);
    try {
        const createDataItemSigner = () => async ({ data, tags = [], target, anchor }) => {
            await (0, exports.checkArPermissions)([
                'ACCESS_ADDRESS',
                'ACCESS_ALL_ADDRESSES',
                'ACCESS_PUBLIC_KEY',
                'SIGN_TRANSACTION',
                'SIGNATURE'
            ]);
            const signed = await window.arweaveWallet.signDataItem({
                data,
                tags,
                anchor,
                target
            });
            const dataItem = new arseeding_arbundles_1.DataItem(Buffer.from(signed));
            return {
                id: await dataItem.id,
                raw: await dataItem.getRaw()
            };
        };
        const signer = createDataItemSigner();
        const messageID = await ao.message({
            process,
            signer,
            tags: replaceTags || [
                { name: 'Action', value: 'Transfer' },
                {
                    name: 'Recipient',
                    value: recipient
                },
                { name: 'Quantity', value: amount },
                ...addTags
            ]
        });
        return messageID;
    }
    catch (err) {
        console.log('err', err);
        throw err;
    }
};
exports.sendAoTransfer = sendAoTransfer;
const transferAsync = async (arJWK, chainType, { symbol, token, from, to, value }) => {
    const arweave = arweave_1.default.init(options);
    let transactionTransfer;
    if ((0, util_1.isArweaveAOSTestToken)(token)) {
        const tokenID = (0, util_1.getTokenAddrByChainType)(token, types_1.ChainType.aostest);
        const transferID = await (0, exports.sendAoTransfer)(tokenID, to, value.toString());
        console.log('transferID', transferID);
        return {
            id: transferID,
            status: 200,
            data: {}
        };
    }
    if (symbol.toUpperCase() === 'AR') {
        transactionTransfer = await arweave.createTransaction({
            target: to,
            quantity: value.toString()
        }, arJWK);
        // PST Token
    }
    else {
        const tokenID = (0, util_1.getTokenAddrByChainType)(token, types_1.ChainType.arweave);
        transactionTransfer = await arweave.createTransaction({
            data: (Math.random() * 10000).toFixed(),
            last_tx: (0, util_1.isArweaveL2PSTTokenSymbol)(token.symbol) ? 'p7vc1iSP6bvH_fCeUFa9LqoV5qiyW-jdEKouAT0XMoSwrNraB9mgpi29Q10waEpO' : undefined,
            reward: (0, util_1.isArweaveL2PSTTokenSymbol)(token.symbol) ? '0' : undefined
        }, arJWK);
        transactionTransfer.addTag('App-Name', 'SmartWeaveAction');
        transactionTransfer.addTag('App-Version', '0.3.0');
        transactionTransfer.addTag('Contract', tokenID);
        transactionTransfer.addTag('Input', JSON.stringify({
            function: 'transfer',
            qty: value.toNumber(),
            target: to
        }));
    }
    if (arJWK === 'use_wallet') {
        try {
            const existingPermissions = await window.arweaveWallet.getPermissions();
            if (!existingPermissions.includes('SIGN_TRANSACTION')) {
                await window.arweaveWallet.connect(['SIGN_TRANSACTION']);
            }
        }
        catch (_a) {
            // Permission is already granted
        }
        const signedTransaction = await window.arweaveWallet.sign(transactionTransfer);
        // TODO: Temp fix arConnect modify reward
        transactionTransfer.reward = signedTransaction.reward;
        transactionTransfer.setSignature({
            id: signedTransaction.id,
            owner: signedTransaction.owner,
            tags: signedTransaction.tags,
            signature: signedTransaction.signature
        });
    }
    else {
        // 直接给原来 transaction 赋值了 signature 值
        await arweave.transactions.sign(transactionTransfer, arJWK);
    }
    let responseTransfer = null;
    if ((0, util_1.isArweaveL2PSTTokenSymbol)(token.symbol)) {
        await (0, api_1.sendRequest)({
            url: 'https://gw.warp.cc/gateway/sequencer/register',
            data: transactionTransfer,
            headers: {
                // 'Accept-Encoding': 'gzip, deflate, br',
                'Content-Type': 'application/json',
                Accept: 'application/json'
            },
            method: 'POST'
        });
        responseTransfer = {
            status: 200,
            data: {}
        };
        // responseTransfer = await fetch('https://gateway.warp.cc/gateway/sequencer/register', {
        //   method: 'POST',
        //   body: JSON.stringify(transactionTransfer),
        //   headers: {
        //     'Accept-Encoding': 'gzip, deflate, br',
        //     'Content-Type': 'application/json',
        //     Accept: 'application/json'
        //   }
        // })
    }
    else {
        responseTransfer = await arweave.transactions.post(transactionTransfer);
    }
    if (responseTransfer.status === 200) {
        // eslint-disable-next-line @typescript-eslint/strict-boolean-expressions
        if (responseTransfer.data.error) {
            throw new Error(responseTransfer.data.error);
        }
        return transactionTransfer;
    }
    throw new Error(ERRORS.TRANSACTION_POST_ERROR);
};
exports.default = {
    signMessageAsync,
    transferAsync
};
