"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRpId = void 0;
const arweave_1 = __importDefault(require("arweave"));
const errors_1 = require("../utils/errors");
const api_1 = require("../api");
const config_1 = require("../config");
const util_1 = require("../utils/util");
const getRpId = () => {
    const domain = document.domain;
    if (domain === 'localhost') {
        return domain;
    }
    if (domain.includes('everpay.io')) {
        return 'everpay.io';
    }
    return domain;
};
exports.getRpId = getRpId;
const signRegisterAsync = async (debug, isSmartAccount, email, everHash, accountData, attachment, expiration) => {
    const everId = (0, util_1.genEverId)(email);
    // expiration 用于添加多密钥时，避免 userId 相同，Safari 14.4+ excludeCredentials bug 导致覆盖丢失密钥
    // https://forums.developer.apple.com/forums/thread/749232?login=true&page=1#785836022
    const userId = (0, util_1.getUserId)(debug, everId, expiration);
    const arr = accountData && accountData.publicValues ? Object.entries(accountData.publicValues) : [];
    const credential = await navigator.credentials.create({
        publicKey: {
            rp: {
                name: 'everpay',
                id: (0, exports.getRpId)()
            },
            user: {
                name: email,
                displayName: email,
                id: arweave_1.default.utils.b64UrlToBuffer(userId)
            },
            challenge: arweave_1.default.utils.b64UrlToBuffer(window.btoa(everHash)),
            attestation: 'direct',
            pubKeyCredParams: [
                {
                    type: 'public-key',
                    alg: -7
                },
                {
                    type: 'public-key',
                    alg: -257
                }
            ],
            // 排除已经注册过的
            excludeCredentials: arr.map((publicIdValueArr) => {
                const id = publicIdValueArr[0];
                return {
                    type: 'public-key',
                    id: arweave_1.default.utils.b64UrlToBuffer(id),
                    transports: [
                        'internal',
                        'usb',
                        'nfc',
                        'ble'
                    ].concat(!util_1.isMobile ? ['hybrid'] : [])
                };
            }),
            timeout: 300000,
            authenticatorSelection: {
                requireResidentKey: true,
                residentKey: 'required',
                userVerification: 'required',
                ...(attachment === 'platform' ? {
                    authenticatorAttachment: 'platform',
                } : {})
            }
        }
    });
    if (credential === null) {
        throw new Error('cancelled');
    }
    const sigJson = {
        id: credential.id,
        rawId: arweave_1.default.utils.bufferTob64Url(credential.rawId),
        attestationObject: arweave_1.default.utils.bufferTob64Url(credential.response.attestationObject),
        clientDataJSON: arweave_1.default.utils.bufferTob64Url(credential.response.clientDataJSON),
        userId
    };
    const signResult = window.btoa(JSON.stringify(sigJson));
    return `${signResult},,FIDO2`;
};
const signMessageAsync = async (debug, isSmartAccount, email, everHash, accountData) => {
    if (accountData == null) {
        const everpayHost = (0, config_1.getEverpayHost)(debug);
        const everId = (0, util_1.genEverId)(email);
        accountData = await (0, api_1.getAccountData)(everpayHost, everId);
    }
    const arr = Object.entries(accountData.publicValues);
    const publicKeyData = {
        allowCredentials: arr.map((publicIdValueArr) => {
            const id = publicIdValueArr[0];
            return {
                type: 'public-key',
                id: arweave_1.default.utils.b64UrlToBuffer(id),
                transports: [
                    'internal',
                    'usb',
                    'nfc',
                    'ble'
                ].concat(!util_1.isMobile ? ['hybrid'] : [])
            };
        })
    };
    const assertion = await navigator.credentials.get({
        publicKey: {
            ...publicKeyData,
            timeout: 300000,
            userVerification: 'required',
            challenge: arweave_1.default.utils.b64UrlToBuffer(window.btoa(everHash)),
            rpId: (0, exports.getRpId)()
        }
    });
    if (assertion === null) {
        throw new Error('cancelled');
    }
    const authenticatorData = assertion.response.authenticatorData;
    const clientDataJSON = assertion.response.clientDataJSON;
    const rawId = assertion.rawId;
    const signature = assertion.response.signature;
    const userHandle = assertion.response.userHandle;
    const sigJson = {
        id: assertion === null || assertion === void 0 ? void 0 : assertion.id,
        rawId: arweave_1.default.utils.bufferTob64Url(rawId),
        clientDataJSON: arweave_1.default.utils.bufferTob64Url(clientDataJSON),
        authenticatorData: arweave_1.default.utils.bufferTob64Url(authenticatorData),
        signature: arweave_1.default.utils.bufferTob64Url(signature),
        userHandle: arweave_1.default.utils.bufferTob64Url(userHandle)
    };
    const sig = window.btoa(JSON.stringify(sigJson));
    return `${sig},${accountData.publicValues[assertion === null || assertion === void 0 ? void 0 : assertion.id]},FIDO2`;
};
const transferAsync = async (isSmartAccount, chainType, { symbol, token, from, to, value }) => {
    throw new Error(errors_1.ERRORS.SMART_ACCOUNT_DEPOSIT_NOT_SUPPORT);
};
exports.default = {
    signRegisterAsync,
    signMessageAsync,
    transferAsync
};
