import ethereumLib from './ethereum';
import arweaveLib from './arweave';
import smartAccountLib from './smartAccount';
import { ChainType } from '../types';
import { checkSignConfig } from '../utils/check';
import { ERRORS } from '../utils/errors';
import hashPersonalMessage from './hashPersonalMessage';
import { isArweaveAOSTestToken, isNodeJs, isPermaswapHaloTokenSymbol } from '../utils/util';
import { openPopup, runPopup } from './popup';
export const getDepositAddr = (info, accountChainType, token) => {
    const symbol = token.symbol;
    if (isArweaveAOSTestToken(token)) {
        return info === null || info === void 0 ? void 0 : info.lockers.aostest;
    }
    if (isPermaswapHaloTokenSymbol(symbol)) {
        return info === null || info === void 0 ? void 0 : info.lockers.psntest;
    }
    if (accountChainType === ChainType.ethereum) {
        return info === null || info === void 0 ? void 0 : info.lockers.ethereum;
    }
    else if (accountChainType === ChainType.arweave) {
        // AR 大小写敏感
        return info === null || info === void 0 ? void 0 : info.lockers.arweave;
    }
    else if (accountChainType === ChainType.moon) {
        return info === null || info === void 0 ? void 0 : info.lockers.moon;
    }
    else if (accountChainType === ChainType.conflux) {
        return info === null || info === void 0 ? void 0 : info.lockers.conflux;
    }
    else if (accountChainType === ChainType.bsc) {
        return info === null || info === void 0 ? void 0 : info.lockers.bsc;
    }
    else if (accountChainType === ChainType.platon) {
        return info === null || info === void 0 ? void 0 : info.lockers.platon;
    }
    else if (accountChainType === ChainType.mapo) {
        return info === null || info === void 0 ? void 0 : info.lockers.mapo;
    }
    throw new Error(ERRORS.INVALID_ACCOUNT_TYPE);
};
export const getEverpayTxMessage = (everpayTxWithoutSig) => {
    const keys = [
        'tokenSymbol',
        'action',
        'from',
        'to',
        'amount',
        'fee',
        'feeRecipient',
        'nonce',
        'tokenID',
        'chainType',
        'chainID',
        'data',
        'version'
    ];
    return keys.map(key => `${key}:${everpayTxWithoutSig[key]}`).join('\n');
};
export const signRegisterAsync = async (config, messageData, attachment) => {
    var _a;
    const from = config.account;
    const accountChainType = config.chainType;
    const personalMsgHashBuffer = hashPersonalMessage(Buffer.from(messageData));
    const personalMsgHex = `0x${personalMsgHashBuffer.toString('hex')}`;
    let sig = '';
    checkSignConfig(accountChainType, config);
    if ((_a = config.isSmartAccount) !== null && _a !== void 0 ? _a : false) {
        sig = await smartAccountLib.signRegisterAsync(Boolean(config.debug), Boolean(config.isSmartAccount), from, personalMsgHex, undefined, attachment);
    }
    else if ([
        ChainType.ethereum,
        ChainType.moon,
        ChainType.conflux,
        ChainType.bsc,
        ChainType.platon,
        ChainType.mapo
    ].includes(accountChainType)) {
        sig = await ethereumLib.signMessageAsync(Boolean(config.debug), config.ethConnectedSigner, from, messageData);
        sig = `${sig},,ECDSA`;
    }
    else if (accountChainType === ChainType.arweave) {
        sig = await arweaveLib.signMessageAsync(Boolean(config.debug), config.arJWK, from, messageData);
        sig = `${sig},RSA`;
    }
    else {
        throw new Error(ERRORS.INVALID_ACCOUNT_TYPE);
    }
    return { everHash: personalMsgHex, sig };
};
export const signMessageAsync = async (config, messageData, accountData, directly) => {
    var _a, _b;
    const from = config.account;
    const accountChainType = config.chainType;
    const personalMsgHashBuffer = hashPersonalMessage(Buffer.from(messageData));
    const personalMsgHex = `0x${personalMsgHashBuffer.toString('hex')}`;
    let sig = '';
    checkSignConfig(accountChainType, config);
    if (!isNodeJs() && Boolean(config.isSmartAccount) && !window.location.host.includes('everpay.io')) {
        const url = `https://app${((_a = config.debug) !== null && _a !== void 0 ? _a : false) ? '-dev' : ''}.everpay.io/sign?account=${config.account}&host=${encodeURIComponent(window.location.host)}&directly=${directly ? '1' : '0'}`;
        // const url = `http://localhost:8080/sign?account=${config.account as string}&host=${encodeURIComponent(window.location.host)}&directly=${directly ? '1' : '0'}`
        const popup = await openPopup(url);
        // 先接收到子窗口的 signPageLoaded 事件
        await runPopup({
            popup,
            type: 'signPageLoaded',
            keepPopup: true
        });
        // 再向子窗口发送 messageData
        popup.postMessage(JSON.stringify({ data: encodeURIComponent(messageData), type: 'signPageMessageData' }), '*');
        sig = await runPopup({
            popup,
            type: 'sign'
        });
    }
    else {
        if ((_b = config.isSmartAccount) !== null && _b !== void 0 ? _b : false) {
            sig = await smartAccountLib.signMessageAsync(Boolean(config.debug), Boolean(config.isSmartAccount), from, directly ? messageData : personalMsgHex, accountData);
        }
        else if ([
            ChainType.ethereum,
            ChainType.moon,
            ChainType.conflux,
            ChainType.bsc,
            ChainType.platon,
            ChainType.mapo
        ].includes(accountChainType)) {
            sig = await ethereumLib.signMessageAsync(Boolean(config.debug), config.ethConnectedSigner, from, messageData);
        }
        else if (accountChainType === ChainType.arweave) {
            sig = await arweaveLib.signMessageAsync(Boolean(config.debug), config.arJWK, from, messageData);
        }
        else {
            throw new Error(ERRORS.INVALID_ACCOUNT_TYPE);
        }
    }
    return { everHash: personalMsgHex, sig };
};
export const getRedPackTxMessage = (redPackTxSig) => {
    const keys = [
        'redpacketUUID',
        'claimBy',
        'salt',
        'createdAt'
    ];
    return keys.map(key => `${key}:${redPackTxSig[key]}`).join('\n');
};
export const transferAsync = async (config, info, params) => {
    checkSignConfig(config.chainType, config);
    const to = getDepositAddr(info, config.chainType, params.token);
    const paramsMergedTo = { ...params, to };
    if ([
        ChainType.ethereum,
        ChainType.moon,
        ChainType.conflux,
        ChainType.bsc,
        ChainType.platon,
        ChainType.mapo
    ].includes(config.chainType)) {
        return await ethereumLib.transferAsync(config.ethConnectedSigner, config.chainType, paramsMergedTo);
    }
    else if (config.chainType === ChainType.arweave) {
        return await arweaveLib.transferAsync(config.arJWK, config.chainType, paramsMergedTo);
    }
    throw new Error(ERRORS.INVALID_ACCOUNT_TYPE);
};
