import Arweave from 'arweave';
import { ERRORS } from '../utils/errors';
import { getAccountData } from '../api';
import { getEverpayHost } from '../config';
import { genEverId, getUserId, isMobile } from '../utils/util';
export const getRpId = () => {
    const domain = document.domain;
    if (domain === 'localhost') {
        return domain;
    }
    if (domain.includes('everpay.io')) {
        return 'everpay.io';
    }
    return domain;
};
const signRegisterAsync = async (debug, isSmartAccount, email, everHash, accountData, attachment, expiration) => {
    const everId = genEverId(email);
    // expiration 用于添加多密钥时，避免 userId 相同，Safari 14.4+ excludeCredentials bug 导致覆盖丢失密钥
    // https://forums.developer.apple.com/forums/thread/749232?login=true&page=1#785836022
    const userId = getUserId(debug, everId, expiration);
    const arr = accountData && accountData.publicValues ? Object.entries(accountData.publicValues) : [];
    const credential = await navigator.credentials.create({
        publicKey: {
            rp: {
                name: 'everpay',
                id: getRpId()
            },
            user: {
                name: email,
                displayName: email,
                id: Arweave.utils.b64UrlToBuffer(userId)
            },
            challenge: Arweave.utils.b64UrlToBuffer(window.btoa(everHash)),
            attestation: 'direct',
            pubKeyCredParams: [
                {
                    type: 'public-key',
                    alg: -7
                },
                {
                    type: 'public-key',
                    alg: -257
                }
            ],
            // 排除已经注册过的
            excludeCredentials: arr.map((publicIdValueArr) => {
                const id = publicIdValueArr[0];
                return {
                    type: 'public-key',
                    id: Arweave.utils.b64UrlToBuffer(id),
                    transports: [
                        'internal',
                        'usb',
                        'nfc',
                        'ble'
                    ].concat(!isMobile ? ['hybrid'] : [])
                };
            }),
            timeout: 300000,
            authenticatorSelection: {
                requireResidentKey: true,
                residentKey: 'required',
                userVerification: 'required',
                ...(attachment === 'platform' ? {
                    authenticatorAttachment: 'platform',
                } : {})
            }
        }
    });
    if (credential === null) {
        throw new Error('cancelled');
    }
    const sigJson = {
        id: credential.id,
        rawId: Arweave.utils.bufferTob64Url(credential.rawId),
        attestationObject: Arweave.utils.bufferTob64Url(credential.response.attestationObject),
        clientDataJSON: Arweave.utils.bufferTob64Url(credential.response.clientDataJSON),
        userId
    };
    const signResult = window.btoa(JSON.stringify(sigJson));
    return `${signResult},,FIDO2`;
};
const signMessageAsync = async (debug, isSmartAccount, email, everHash, accountData) => {
    if (accountData == null) {
        const everpayHost = getEverpayHost(debug);
        const everId = genEverId(email);
        accountData = await getAccountData(everpayHost, everId);
    }
    const arr = Object.entries(accountData.publicValues);
    const publicKeyData = {
        allowCredentials: arr.map((publicIdValueArr) => {
            const id = publicIdValueArr[0];
            return {
                type: 'public-key',
                id: Arweave.utils.b64UrlToBuffer(id),
                transports: [
                    'internal',
                    'usb',
                    'nfc',
                    'ble'
                ].concat(!isMobile ? ['hybrid'] : [])
            };
        })
    };
    const assertion = await navigator.credentials.get({
        publicKey: {
            ...publicKeyData,
            timeout: 300000,
            userVerification: 'required',
            challenge: Arweave.utils.b64UrlToBuffer(window.btoa(everHash)),
            rpId: getRpId()
        }
    });
    if (assertion === null) {
        throw new Error('cancelled');
    }
    const authenticatorData = assertion.response.authenticatorData;
    const clientDataJSON = assertion.response.clientDataJSON;
    const rawId = assertion.rawId;
    const signature = assertion.response.signature;
    const userHandle = assertion.response.userHandle;
    const sigJson = {
        id: assertion === null || assertion === void 0 ? void 0 : assertion.id,
        rawId: Arweave.utils.bufferTob64Url(rawId),
        clientDataJSON: Arweave.utils.bufferTob64Url(clientDataJSON),
        authenticatorData: Arweave.utils.bufferTob64Url(authenticatorData),
        signature: Arweave.utils.bufferTob64Url(signature),
        userHandle: Arweave.utils.bufferTob64Url(userHandle)
    };
    const sig = window.btoa(JSON.stringify(sigJson));
    return `${sig},${accountData.publicValues[assertion === null || assertion === void 0 ? void 0 : assertion.id]},FIDO2`;
};
const transferAsync = async (isSmartAccount, chainType, { symbol, token, from, to, value }) => {
    throw new Error(ERRORS.SMART_ACCOUNT_DEPOSIT_NOT_SUPPORT);
};
export default {
    signRegisterAsync,
    signMessageAsync,
    transferAsync
};
