import { isAddress } from '@ethersproject/address';
import isString from 'lodash/isString';
import { v4 as uuidv4 } from 'uuid';
import BN from 'bignumber.js';
import { ERRORS } from './errors';
import { ChainType } from '../types';
import { bundleInternalTxVersion } from '../config';
import sha256 from 'crypto-js/sha256';
import encHex from 'crypto-js/enc-hex';
BN.config({
    EXPONENTIAL_AT: 1000,
});
export const toBN = (x) => {
    if (isNaN(Number(x)))
        return new BN(0);
    if (x instanceof BN)
        return x;
    if (typeof x === 'string') {
        if (x.indexOf('0x') === 0 || x.indexOf('-0x') === 0) {
            return new BN((x).replace('0x', ''), 16);
        }
    }
    return new BN(x);
};
export const fromUnitToDecimalBN = (x, decimals) => {
    return toBN(x).times(toBN(10).pow(decimals));
};
export const fromUnitToDecimal = (x, decimals) => {
    return fromUnitToDecimalBN(x, decimals).toString();
};
export const fromDecimalToUnitBN = (x, decimals) => {
    return toBN(x).dividedBy(toBN(10).pow(decimals));
};
export const fromDecimalToUnit = (x, decimals) => {
    return fromDecimalToUnitBN(x, decimals).toString();
};
export const getTimestamp = () => Math.round(Date.now() / 1000);
export const getTokenByTag = (tag, tokenList) => {
    return tokenList === null || tokenList === void 0 ? void 0 : tokenList.find(t => matchTokenTag(genTokenTag(t), tag));
};
export const isEthereumAddress = isAddress;
export const isArweaveAddress = (address) => {
    return isString(address) && address.length === 43 && address.search(/[a-z0-9A-Z_-]{43}/g) === 0;
};
export const isArweaveChainPSTMode = (token) => {
    if (token == null)
        return false;
    return token.crossChainInfoList[ChainType.arweave] != null && token.symbol.toUpperCase() !== 'AR';
};
export const isArweaveL2PSTTokenSymbol = (symbol) => {
    return ['STAMP', 'U'].includes(symbol === null || symbol === void 0 ? void 0 : symbol.toUpperCase());
};
export const isArweaveAOSTestToken = (token) => {
    var _a;
    return token.chainType.includes(ChainType.aostest) && ['AOCRED', '0RBT', 'TRUNK', 'EXP', 'NEXP', 'EXP(ARIO)', 'BP', 'AR'].includes((_a = token === null || token === void 0 ? void 0 : token.symbol) === null || _a === void 0 ? void 0 : _a.toUpperCase());
};
export const isPermaswapHaloTokenSymbol = (symbol) => {
    return (symbol === null || symbol === void 0 ? void 0 : symbol.toUpperCase()) === 'HALO';
};
export const getAccountChainType = (from) => {
    if (isEthereumAddress(from)) {
        return ChainType.ethereum;
    }
    if (isArweaveAddress(from)) {
        return ChainType.arweave;
    }
    throw new Error(ERRORS.INVALID_ACCOUNT_TYPE);
};
export const getTokenAddrByChainType = (token, chainType) => {
    const crossChainInfo = token.crossChainInfoList[chainType];
    return crossChainInfo.targetTokenId;
};
export const getChainDecimalByChainType = (token, chainType) => {
    const crossChainInfo = token.crossChainInfoList[chainType];
    return crossChainInfo.targetDecimals;
};
export const getTokenBurnFeeByChainType = (token, feeItem, chainType) => {
    return feeItem.burnFeeMap[chainType];
};
export const genTokenTag = (token) => {
    const { chainType, symbol, id } = token;
    const chainTypes = chainType.split(',');
    const tokenAddrs = id.split(',').map((addr, index) => {
        if ([
            ChainType.ethereum,
            ChainType.bsc,
            ChainType.conflux,
            ChainType.moon,
            ChainType.platon,
            'everpay'
        ].includes(chainTypes[index])) {
            return addr.toLowerCase();
        }
        return addr;
    });
    return `${chainType.toLowerCase()}-${symbol.toLowerCase()}-${tokenAddrs.join(',')}`;
};
export const matchTokenTag = (tag1, tag2) => {
    return (tag1 === null || tag1 === void 0 ? void 0 : tag1.toLowerCase()) === (tag2 === null || tag2 === void 0 ? void 0 : tag2.toLowerCase());
};
export const genExpressData = (params) => {
    const { chainType, to, fee } = params;
    return {
        appId: 'express',
        withdrawAction: 'pay',
        withdrawTo: to,
        withdrawChainType: chainType,
        withdrawFee: fee
    };
};
export const genBundleData = (params) => {
    const items = params.items.map((item) => {
        const { tag, amount, from, to, data } = item;
        const token = getTokenByTag(tag, params.tokenList);
        // 注意：顺序必须与后端保持一致，来让 JSON.stringify() 生成的字符串顺序与后端也一致
        return {
            tag: genTokenTag(token),
            chainID: token.chainID,
            from,
            to,
            data: data != null ? data : '',
            amount: fromUnitToDecimal(amount, token.decimals)
        };
    });
    const salt = uuidv4();
    const version = bundleInternalTxVersion;
    return {
        // 注意：顺序必须与后端保持一致，来让 JSON.stringify() 生成的字符串顺序与后端也一致
        items,
        expiration: params.expiration,
        salt,
        version,
        data: params.data != null ? params.data : ''
    };
};
export const isSmartAccount = (account) => {
    return /^\w+([-+.]\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*$/.test(account);
};
export const getUserId = (debug, everId, expiration) => {
    const everpayChainIdStr = debug ? '5' : '1';
    // hash the message
    const hash = sha256(`${everId.trim().toLowerCase()}${everpayChainIdStr}${expiration ? expiration : ''}`);
    const arrBuffer = hexToUint8Array(hash.toString()).slice(0, 10);
    const b64encoded = window.btoa(String.fromCharCode.apply(null, arrBuffer));
    return b64encoded;
};
const checkSum = (idBytes) => {
    const hash = sha256(encHex.parse(uint8ArrayToHex(Uint8Array.from([...Buffer.from('eid'), ...idBytes]))));
    return hexToUint8Array(hash.toString()).slice(0, 2);
};
export const uint8ArrayToHex = (uint8Array) => {
    return [...uint8Array].map((b) => {
        return b.toString(16).padStart(2, '0');
    }).join('');
};
export const hexToUint8Array = (hexString) => Uint8Array.from(hexString.match(/.{1,2}/g).map((byte) => parseInt(byte, 16)));
export const genEverId = (email) => {
    const str = email.toLowerCase().trim();
    const hash = sha256(str);
    const idBytes = hexToUint8Array(hash.toString());
    const sum = checkSum(idBytes);
    const concatArray = Uint8Array.from([...idBytes, ...sum]);
    return `eid${uint8ArrayToHex(concatArray)}`;
};
export const isNodeJs = () => typeof process !== 'undefined' &&
    process.versions != null &&
    process.versions.node != null;
const mobileRE = /(android|bb\d+|meego).+mobile|armv7l|avantgo|bada\/|ipad|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series[46]0|symbian|treo|up\.(browser|link)|vodafone|wap|windows (ce|phone)|xda|xiino/i;
const tabletRE = /android|ipad|playbook|silk/i;
export const isMobileDevice = (opts) => {
    if (opts == null)
        opts = {};
    let ua = opts.ua;
    if (ua == null && typeof navigator !== 'undefined')
        ua = navigator.userAgent;
    if ((ua === null || ua === void 0 ? void 0 : ua.headers) != null && typeof ua.headers['user-agent'] === 'string') {
        ua = ua.headers['user-agent'];
    }
    if (typeof ua !== 'string')
        return false;
    let result = mobileRE.test(ua) || ((opts === null || opts === void 0 ? void 0 : opts.tablet) != null && tabletRE.test(ua));
    if (!result &&
        (opts === null || opts === void 0 ? void 0 : opts.tablet) != null &&
        (opts === null || opts === void 0 ? void 0 : opts.featureDetect) != null &&
        (navigator === null || navigator === void 0 ? void 0 : navigator.maxTouchPoints) > 1 &&
        ua.includes('Macintosh') &&
        ua.includes('Safari')) {
        result = true;
    }
    return result;
};
export const isMobile = isMobileDevice({
    tablet: true,
    featureDetect: true
});
